import logging
import mimetypes
from datetime import date
from io import BytesIO

from drc_cmis.client.mapper import (
    CONNECTION_MAP, DOCUMENT_MAP, REVERSE_CONNECTION_MAP, REVERSE_DOCUMENT_MAP
)

from .utils import CMISRequest

logger = logging.getLogger(__name__)


class CMISBaseObject(CMISRequest):
    def __init__(self, data):
        super().__init__()
        self.data = data
        self.properties = dict(data.get('properties', {}))

    def reload(self):
        logger.debug("CMIS: DRC_DOCUMENT: reload")
        params = {
            "objectId": self.objectId,
            "cmisselector": "object",
        }

        json_response = self.get_request(self.root_folder_url, params=params)
        self.data = json_response
        self.properties = json_response.get('properties')

    def get_object_parents(self):
        logger.debug("CMIS: DRC_DOCUMENT: get_object_parents")
        params = {
            "objectId": self.objectId,
            "cmisselector": "parents",
        }

        json_response = self.get_request(self.root_folder_url, params=params)
        return self.get_all_objects(json_response, Folder)

    def move(self, sourceFolder, targetFolder):
        data = {
            "objectId": self.objectId,
            "cmisaction": "move",
            "sourceFolderId": sourceFolder.objectId,
            "targetFolderId": targetFolder.objectId
        }
        logger.debug(f"From: {sourceFolder.name} To: {targetFolder.name}")
        logger.debug(f"From: {sourceFolder.objectTypeId} To: {targetFolder.objectTypeId}")
        # invoke the URL
        json_response = self.post_request(self.root_folder_url, data=data)
        self.data = json_response
        self.properties = json_response.get('properties')
        return self


class Document(CMISBaseObject):
    def __getattr__(self, name):
        convert_string = f"drc:{name}"
        if name in DOCUMENT_MAP:
            convert_string = DOCUMENT_MAP.get(name)
        elif name in CONNECTION_MAP:
            convert_string = CONNECTION_MAP.get(name)
        elif convert_string not in REVERSE_CONNECTION_MAP and convert_string not in REVERSE_DOCUMENT_MAP:
            convert_string = f"cmis:{name}"
        return self.properties.get(convert_string, {}).get('value')

    def checkout(self):
        logger.debug("CMIS: DRC_DOCUMENT: checkout")
        data = {"objectId": self.objectId, "cmisaction": "checkOut"}
        json_response = self.post_request(self.root_folder_url, data=data)
        return Document(json_response)

    def update_properties(self, properties):
        logger.debug("CMIS: DRC_DOCUMENT: update_properties")
        data = {
            "objectId": self.objectId,
            "cmisaction": "update",
        }
        prop_count = 0
        for prop_key, prop_value in properties.items():
            # Skip property because update is not allowed
            if prop_key == 'cmis:objectTypeId':
                continue

            if isinstance(prop_value, date):
                prop_value = prop_value.strftime('%Y-%m-%dT%H:%I:%S.000Z')

            data["propertyId[%s]" % prop_count] = prop_key
            data["propertyValue[%s]" % prop_count] = prop_value
            prop_count += 1

        # invoke the URL
        json_response = self.post_request(self.root_folder_url, data=data)
        self.data = json_response
        self.properties = json_response.get('properties')

        return self

    def get_private_working_copy(self):
        logger.debug("CMIS: DRC_DOCUMENT: get_private_working_copy")
        self.properties["cmis:objectId"]["value"] = self.properties["cmis:versionSeriesCheckedOutId"]["value"]
        return self

    def checkin(self, checkin_comment, major=True):
        logger.debug("CMIS: DRC_DOCUMENT: checkin")
        props = {
            "objectId": self.objectId,
            "cmisaction": "checkIn",
            "checkinComment": checkin_comment,
            "major": major,
        }

        # invoke the URL
        json_response = self.post_request(self.root_folder_url, props)
        return Document(json_response)

    def set_content_stream(self, content_file):
        logger.debug("CMIS: DRC_DOCUMENT: set_content_stream")
        data = {
            "objectId": self.objectId,
            "cmisaction": "setContent",
        }

        mimetype = None
        # need to determine the mime type
        if not mimetype and hasattr(content_file, 'name'):
            mimetype, _encoding = mimetypes.guess_type(content_file.name)

        if not mimetype:
            mimetype = 'application/binary'

        files = {self.name: (self.name, content_file, mimetype)}

        json_response = self.post_request(self.root_folder_url, data=data, files=files)
        return Document(json_response)

    def get_content_stream(self) -> BytesIO:
        logger.debug("CMIS: DRC_DOCUMENT: get_content_stream")
        params = {
            "objectId": self.objectId,
            "cmisaction": "content",
        }
        file_content = self.get_request(self.root_folder_url, params=params)
        return BytesIO(file_content)

    def get_all_versions(self, **kwargs):
        logger.debug("CMIS: DRC_DOCUMENT: checkout")
        params = {"objectId": self.objectId, "cmisselector": "versions"}
        json_response = self.get_request(self.root_folder_url, params=params)
        return self.parse_versions(json_response)

    def parse_versions(self, json):
        versions = {}
        for json_version in json:
            doc = Document(json_version)
            versions[doc.versionLabel] = doc
        return versions


class Folder(CMISBaseObject):
    def __getattr__(self, name):
        return self.properties.get(f"cmis:{name}", {}).get('value')

    def get_children(self):
        logger.debug("CMIS: DRC_DOCUMENT: get_children")
        data = {
            "cmisaction": "query",
            "statement": f"SELECT * FROM cmis:folder WHERE IN_FOLDER('{self.objectId}')",
        }
        json_response = self.post_request(self.base_url, data=data)
        return self.get_all_resutls(json_response, Folder)

    def create_folder(self, name, properties={}, **kwargs):
        logger.debug("CMIS: DRC_DOCUMENT: create_folder")
        data = {
            "objectId": self.objectId,
            "cmisaction": "createFolder",
            "propertyId[0]": "cmis:name",
            "propertyValue[0]": name
        }

        data["propertyId[1]"] = "cmis:objectTypeId"
        if 'cmis:objectTypeId' in properties.keys():
            data["propertyValue[1]"] = properties['cmis:objectTypeId']
        else:
            data["propertyValue[1]"] = "cmis:folder"

        prop_count = 2
        for prop, value in properties.items():
            data[f"propertyId[{prop_count}]"] = prop
            data[f"propertyValue[{prop_count}]"] = value
            prop_count += 1

        json_response = self.post_request(self.root_folder_url, data=data)
        return Folder(json_response)

    def create_document(self, name, properties, content_file=None, **kwargs):
        logger.debug("CMIS: DRC_DOCUMENT: create_document")
        data = {
            "objectId": self.objectId,
            "cmisaction": "createDocument",
            "propertyId[0]": "cmis:name",
            "propertyValue[0]": name
        }

        data["propertyId[1]"] = "cmis:objectTypeId"
        if 'cmis:objectTypeId' in properties.keys():
            data["propertyValue[1]"] = properties['cmis:objectTypeId']
        else:
            data["propertyValue[1]"] = "drc:document"

        prop_count = 2
        for prop_key, prop_value in properties.items():
            if isinstance(prop_value, date):
                prop_value = prop_value.strftime('%Y-%m-%dT%H:%I:%S.000Z')

            data[f"propertyId[{prop_count}]"] = prop_key
            data[f"propertyValue[{prop_count}]"] = prop_value
            prop_count += 1

        json_response = self.post_request(self.root_folder_url, data=data)
        cmis_doc = Document(json_response)
        return cmis_doc.set_content_stream(content_file)

    def delete_tree(self, **kwargs):
        data = {
            "objectId": self.objectId,
            "cmisaction": "deleteTree"
        }
        self.post_request(self.root_folder_url, data=data)



# Generated by Django 2.0.13 on 2019-05-17 07:54

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="CMISConfig",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("client_url", models.URLField(default="http://localhost:8082/alfresco/cmisatom")),
                ("client_user", models.CharField(default="admin", max_length=200)),
                ("client_password", models.CharField(default="admin", max_length=200)),
            ],
            options={"verbose_name": "CMIS Configuration"},
        )
    ]



class CMISQuery:
    """
    Small, not feature-complete utility class for building CMIS queries with
    escaping built in.

    Usage:
    >>> query = CMSQuery("SELECT * FROM cmis:document WHERE cmis:objectTypeId = '%s'")
    >>> query('drc:document')
    "SELECT * FROM cmis:document WHERE cmis:objectTypeId = 'drc:document';"
    """

    def __init__(self, query):
        self.query = query

    def __call__(self, *args):
        args = tuple((self.escape(arg) for arg in args))
        return self.query % args

    def escape(self, value):
        """
        Escapes the characters in value for the CMIS queries.

        Poor documentation references:
          * https://community.alfresco.com/docs/DOC-5898-cmis-query-language#Literals
          * http://docs.alfresco.com/community/concepts/rm-searchsyntax-literals.html
        """
        # if isinstance(value, uuid.UUID):
        #     value = str(value).replace('-', '').replace('-', '').replace('-', '').replace('-', '')

        if isinstance(value, str):
            value = value.replace("'", "\\'")
            value = value.replace('"', '\\"')
        return value



import logging
import random
import string
from datetime import datetime
from decimal import Decimal
from io import BytesIO

from django.conf import settings

from cmislib.exceptions import UpdateConflictException

from drc_cmis.cmis.drc_document import Document, Folder
from drc_cmis.cmis.utils import CMISRequest

from .exceptions import (
    DocumentConflictException, DocumentDoesNotExistError, DocumentExistsError,
    DocumentLockConflictException, DocumentNotLockedException,
    GetFirstException
)
from .mapper import mapper
from .query import CMISQuery

logger = logging.getLogger(__name__)


class CMISDRCClient(CMISRequest):
    """
    DRC client implementation using the CMIS protocol.

    This class makes it easier to query for everything you need.
    """

    documents_in_folders_query = CMISQuery("SELECT * FROM drc:document WHERE %s")
    find_folder_by_case_url_query = CMISQuery("SELECT * FROM drc:zaakfolder WHERE drc:zaak__url='%s'")

    document_via_identification_query = CMISQuery(
        "SELECT * FROM drc:document WHERE drc:document__identificatie = '%s' %s"
    )
    document_via_cmis_id_query = CMISQuery(
        "SELECT * FROM drc:document WHERE cmis:objectId = 'workspace://SpacesStore/%s;1.0' %s"
    )

    _repo = None
    _root_folder = None
    _base_folder = None

    @property
    def _get_base_folder(self):
        logger.debug("CMIS_CLIENT: _get_base_folder")
        if not self._base_folder:
            base = self.get_request(self.root_folder_url)
            for folder_response in base.get('objects'):
                folder = Folder(folder_response['object'])
                if folder.name == settings.BASE_FOLDER_LOCATION:
                    self._base_folder = folder
            if not self._base_folder:
                folder = Folder({})
                self._base_folder = folder.create_folder(name=settings.BASE_FOLDER_LOCATION)
        return self._base_folder

    # ZRC Notification client calls.
    def get_or_create_zaaktype_folder(self, zaaktype):
        """
        Create a folder with the prefix 'zaaktype-' to make a zaaktype folder

        """
        logger.debug("CMIS_CLIENT: get_or_create_zaaktype_folder")
        properties = {
            "cmis:objectTypeId": "F:drc:zaaktypefolder",
            mapper("url", "zaaktype"): zaaktype.get("url"),
            mapper("identificatie", "zaaktype"): zaaktype.get("identificatie"),
        }

        folder_name = f"zaaktype-{zaaktype.get('omschrijving')}-{zaaktype.get('identificatie')}"
        cmis_folder = self._get_or_create_folder(folder_name, self._get_base_folder, properties)
        return cmis_folder

    def get_or_create_zaak_folder(self, zaak, zaaktype_folder):
        """
        Create a folder with the prefix 'zaak-' to make a zaak folder

        """
        logger.debug("CMIS_CLIENT: get_or_create_zaak_folder")
        properties = {
            "cmis:objectTypeId": "F:drc:zaakfolder",
            mapper("url", "zaak"): zaak.get("url"),
            mapper("identificatie", "zaak"): zaak.get("identificatie"),
            mapper("zaaktype", "zaak"): zaak.get("zaaktype"),
            mapper("bronorganisatie", "zaak"): zaak.get("bronorganisatie"),
        }
        cmis_folder = self._get_or_create_folder(f"zaak-{zaak.get('identificatie')}", zaaktype_folder, properties)
        return cmis_folder

    # DRC client calls.
    def create_document(self, identification, data, content=None):
        """
        Create a cmis document.

        Args:
            identification (str): A unique identifier for the document.
            data (dict): A dict with all the data that needs to be saved on the document.
            content (BytesStream): The content of the document.

        Returns:
            AtomPubDocument: A cmis document.

        Raises:
            DocumentExistsError: when a document already uses the same identifier.

        """
        logger.debug("CMIS_CLIENT: create_document")
        self._check_document_exists(identification)

        now = datetime.now()

        if content is None:
            content = BytesIO()

        year_folder = self._get_or_create_folder(str(now.year), self._get_base_folder)
        month_folder = self._get_or_create_folder(str(now.month), year_folder)
        day_folder = self._get_or_create_folder(str(now.day), month_folder)
        properties = self._build_properties(identification, data)

        # Make sure that the content is set.
        content.seek(0)
        return day_folder.create_document(
            name=properties.pop("cmis:name"),
            properties=properties,
            content_file=content,
        )

    def get_cmis_documents(self, filters=None, page=1, results_per_page=100):
        """
        Gives a list of cmis documents.

        Args:
            filters (dict): A dict of filters that need to be applied to the search.

        Returns:
            AtomPubDocument: A list of CMIS documents.

        """
        logger.debug("CMIS_CLIENT: get_cmis_documents")
        filter_string = self._build_filter(filters, strip_end=True)
        query = f"SELECT * FROM drc:document WHERE drc:document__verwijderd='false'"
        if filter_string:
            query += f' AND {filter_string}'

        print(query)
        data = {
            "cmisaction": "query",
            "statement": query,
        }

        skip_count = 0
        if page:
            results_per_page = results_per_page
            max_items = results_per_page
            skip_count = page * results_per_page - results_per_page

            data["maxItems"] = max_items
            data["skipCount"] = skip_count

        json_response = self.post_request(self.base_url, data)
        print(json_response)
        results = self.get_all_resutls(json_response, Document)
        return {
            'has_next': json_response['hasMoreItems'],
            'total_count': json_response['numItems'],
            'has_prev': skip_count > 0,
            'results': results,
        }

    def get_cmis_document(self, identification, via_identification=None, filters=None):
        """
        Given a cmis document instance.

        :param identification.
        :return: :class:`AtomPubDocument` object
        """
        logger.debug("CMIS_CLIENT: get_cmis_document")
        document_query = self.document_via_cmis_id_query
        if via_identification:
            document_query = self.document_via_identification_query
        filter_string = self._build_filter(filters, filter_string="AND ", strip_end=True)
        query = document_query(identification, filter_string)
        data = {
            "cmisaction": "query",
            "statement": query,
        }
        json_response = self.post_request(self.base_url, data)
        try:
            return self.get_first_result(json_response, Document)
        except GetFirstException:
            error_string = f"Document met identificatie {identification} bestaat niet in het CMIS connection"
            raise DocumentDoesNotExistError(error_string)

    def update_cmis_document(self, uuid, lock, data, content=None):
        logger.debug("CMIS_CLIENT: update_cmis_document")
        cmis_doc = self.get_cmis_document(uuid)
        if not cmis_doc.versionSeriesCheckedOutId:
            raise DocumentNotLockedException("Document is niet gelocked.")
        if lock != cmis_doc.versionSeriesCheckedOutId:
            raise DocumentLockConflictException("Wrong lock given")

        pwc = cmis_doc.get_private_working_copy()

        # build up the properties
        current_properties = cmis_doc.properties
        new_properties = self._build_properties(cmis_doc.identification, data)

        diff_properties = {
            key: value
            for key, value in new_properties.items()
            if current_properties.get(key) != value
        }

        try:
            pwc.update_properties(diff_properties)
        except UpdateConflictException as exc:
            # Node locked!
            raise DocumentConflictException from exc

        if content is not None:
            pwc.set_content_stream(content)

        return pwc

    def delete_cmis_document(self, uuid):
        """
        Update the property that the document is deleted.

        Arguments:
            uuid (str): The uuid of the document.

        Returns:
            AtomPubDocument: A CMIS document.

        Raises:
            DocumentConflictException: If the document could not be updated.
        """
        logger.debug("CMIS_CLIENT: delete_cmis_document")
        cmis_doc = self.get_cmis_document(uuid)
        new_properties = {mapper("verwijderd"): True}

        try:
            cmis_doc.update_properties(new_properties)
        except UpdateConflictException as exc:
            # Node locked!
            raise DocumentConflictException from exc
        return cmis_doc

    # Split ########################################################################################

    def update_case_connection(self, uuid, data):
        logger.debug("CMIS_CLIENT: update_case_connection")
        cmis_doc = self.get_cmis_document(uuid)

        current_properties = cmis_doc.properties
        new_properties = self._build_case_properties(data, allow_empty=False)
        diff_properties = {
            key: value
            for key, value in new_properties.items()
            if current_properties.get(key) != new_properties.get(key)
        }

        if diff_properties:
            try:
                cmis_doc.update_properties(diff_properties)
            except UpdateConflictException as exc:
                # Node locked!
                raise DocumentConflictException from exc
        return cmis_doc

    def delete_case_connection(self, uuid):
        logger.debug("CMIS_CLIENT: delete_case_connection")
        cmis_doc = self.get_cmis_document(uuid)
        parents = cmis_doc.get_object_parents()
        connection_count = len(parents)

        in_zaakfolder = False
        for parent in parents:
            if parent.properties.get("cmis:objectTypeId") == "F:drc:zaakfolder":
                in_zaakfolder = True

        if in_zaakfolder and connection_count == 1:
            now = datetime.now()
            year_folder = self._get_or_create_folder(str(now.year), self._get_base_folder)
            month_folder = self._get_or_create_folder(str(now.month), year_folder)
            day_folder = self._get_or_create_folder(str(now.day), month_folder)

            self.move_to_case(cmis_doc, day_folder)

        # Clear properties
        current_properties = cmis_doc.properties
        new_properties = self._build_case_properties({}, allow_empty=True)
        diff_properties = {
            key: value
            for key, value in new_properties.items()
            if current_properties.get(key) != new_properties.get(key)
        }

        if diff_properties:
            try:
                cmis_doc.update_properties(diff_properties)
            except UpdateConflictException as exc:
                # Node locked!
                raise DocumentConflictException from exc
        return cmis_doc

    def move_to_case(self, cmis_doc, folder):
        logger.debug("CMIS_CLIENT: move_to_case")
        parent = [parent for parent in cmis_doc.get_object_parents()][0]
        cmis_doc.move(parent, folder)

    def copy_document(self, cmis_doc, folder, data):
        """
        The copy from source is not supported via the atom pub bindings.
        So we create a new document with the same values.
        """
        logger.debug("CMIS_CLIENT: copy_document")
        properties = {}

        properties.update(**{
            'cmis:objectTypeId': cmis_doc.objectTypeId,
            'drc:document__auteur': cmis_doc.auteur,
            'drc:document__beschrijving': cmis_doc.beschrijving,
            'drc:document__bestandsnaam': cmis_doc.bestandsnaam,
            'drc:document__bronorganisatie': cmis_doc.bronorganisatie,
            'drc:document__creatiedatum': cmis_doc.creatiedatum,
            'drc:document__formaat': cmis_doc.formaat,
            'drc:document__identificatie': cmis_doc.identificatie,
            'drc:document__indicatiegebruiksrecht': cmis_doc.indicatiegebruiksrecht,
            'drc:document__informatieobjecttype': cmis_doc.informatieobjecttype,
            'drc:document__integriteitalgoritme': cmis_doc.integriteitalgoritme,
            'drc:document__integriteitdatum': cmis_doc.integriteitdatum,
            'drc:document__integriteitwaarde': cmis_doc.integriteitwaarde,
            'drc:document__link': cmis_doc.link,
            'drc:document__ondertekeningdatum': cmis_doc.ondertekeningdatum,
            'drc:document__ondertekeningsoort': cmis_doc.ondertekeningsoort,
            'drc:document__ontvangstdatum': cmis_doc.ontvangstdatum,
            'drc:document__status': cmis_doc.status,
            'drc:document__taal': cmis_doc.taal,
            'drc:document__titel': f"{cmis_doc.titel} - copy",
            'drc:document__vertrouwelijkaanduiding': cmis_doc.vertrouwelijkaanduiding,
            'drc:document__verwijderd': cmis_doc.verwijderd,
            'drc:document__verzenddatum': cmis_doc.verzenddatum,
            'drc:kopie_van': cmis_doc.id,  # Keep tack of where this is copied from.
        })

        new_properties = self._build_case_properties(data)
        properties.update(**new_properties)

        if not folder:
            now = datetime.now()
            year_folder = self._get_or_create_folder(str(now.year), self._get_base_folder)
            month_folder = self._get_or_create_folder(str(now.month), year_folder)
            folder = self._get_or_create_folder(str(now.day), month_folder)

        # Update the cmis:name to make it more unique
        file_name = f"{cmis_doc.titel}-{self.get_random_string()}"
        properties['cmis:name'] = file_name

        stream = cmis_doc.get_content_stream()
        new_doc = folder.create_document(
            name=file_name,
            properties=properties,
            content_file=stream,
        )

        return new_doc

    def get_random_string(self, number=6):
        logger.debug("CMIS_CLIENT: get_random_string")
        return ''.join(random.choices(string.ascii_uppercase + string.digits, k=number))

    def get_folder_from_case_url(self, zaak_url):
        logger.debug("CMIS_CLIENT: get_folder_from_case_url")
        query = self.find_folder_by_case_url_query(zaak_url)
        data = {
            "cmisaction": "query",
            "statement": query,
        }

        json_response = self.post_request(self.base_url, data)
        try:
            return self.get_first_result(json_response, Folder)
        except GetFirstException:
            return None

    # Private functions.
    def _get_or_create_folder(self, name, parent, properties=None):
        """
        Get or create the folder with :param:`name` in :param:`parent`.

        :param name: string, the name of the folder to create.
        :param parent: parent folder to create the folder in as subfolder.
        :param properties: dictionary with cmis and/or custom properties to
          pass to the folder object
        :return: the folder that was retrieved or created.
        """
        logger.debug("CMIS_CLIENT: _get_or_create_folder")
        existing = self._get_folder(name, parent)
        if existing:
            return existing
        return parent.create_folder(name, properties=properties or {})

    def _get_folder(self, name, parent):
        logger.debug("CMIS_CLIENT: _get_folder")
        existing = next((child for child in parent.get_children() if str(child.name) == str(name)), None)
        if existing is not None:
            return existing
        return None

    def _build_filter(self, filters, filter_string="", strip_end=False):
        logger.debug("CMIS_CLIENT: _build_filter")
        if filters:
            for key, value in filters.items():
                if mapper(key):
                    key = mapper(key)
                elif mapper(key, type="connection"):
                    key = mapper(key, type="connection")

                if value and value in ["NULL", "NOT NULL"]:
                    filter_string += f"{key} IS {value} AND "
                elif isinstance(value, Decimal):
                    filter_string += f"{key} = {value} AND "
                elif value:
                    filter_string += f"{key} = '{value}' AND "

        if strip_end and filter_string[-4:] == "AND ":
            filter_string = filter_string[:-4]

        return filter_string

    def _build_properties(self, identification, data):
        logger.debug("CMIS_CLIENT: _build_properties")
        base_properties = {mapper(key): value for key, value in data.items() if mapper(key)}
        base_properties["cmis:objectTypeId"] = "D:drc:document"
        base_properties['cmis:name'] = f"{data.get('titel')}-{self.get_random_string()}"
        base_properties[mapper("identificatie")] = identification

        if "cmis:versionLabel" in base_properties:
            del base_properties['cmis:versionLabel']
        return base_properties

    def _build_case_properties(self, data, allow_empty=True):
        logger.debug("CMIS_CLIENT: _build_case_properties")
        props = {}
        if data.get("object") or allow_empty:
            props[mapper("object", "connection")] = data.get("object")
        if data.get("object_type") or allow_empty:
            props[mapper("object_type", "connection")] = data.get("object_type")
        if data.get("aard_relatie") or allow_empty:
            props[mapper("aard_relatie", "connection")] = data.get("aard_relatie")
        if data.get("titel") or allow_empty:
            props[mapper("titel", "connection")] = data.get("titel")
        if data.get("beschrijving") or allow_empty:
            props[mapper("beschrijving", "connection")] = data.get("beschrijving")
        if data.get("registratiedatum") or allow_empty:
            props[mapper("registratiedatum", "connection")] = data.get("registratiedatum")

        return props

    def _check_document_exists(self, identification):
        logger.debug("CMIS_CLIENT: _check_document_exists")
        try:
            self.get_cmis_document(identification, via_identification=True)
        except DocumentDoesNotExistError:
            pass
        else:
            error_string = f"Document identificatie {identification} is niet uniek"
            raise DocumentExistsError(error_string)



# Generated by Django 2.0.13 on 2019-05-23 10:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("drc_cmis", "0001_initial_new")]

    operations = [
        migrations.CreateModel(
            name="CMISFolderLocation",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("location", models.CharField(default="", max_length=200)),
            ],
        ),
        migrations.AddField(model_name="cmisconfig", name="locations", field=models.ManyToManyField(to="drc_cmis.CMISFolderLocation")),
    ]



import base64
import logging
from datetime import datetime
from decimal import Decimal, InvalidOperation

from django.conf import settings
from django.urls import reverse

import iso8601

from .mapper import reverse_mapper

logger = logging.getLogger(__name__)


def to_date(value):
    tmp_datetime = parseDateTimeValue(value)
    if tmp_datetime:
        return tmp_datetime.date()
    return None


def to_datetime(value):
    tmp_datetime = parseDateTimeValue(value)
    return tmp_datetime


def parseDateTimeValue(value):
    """
    Utility function to return a datetime from a string.
    """
    if type(value) == str:
        return iso8601.parse_date(value)
    elif type(value) == int:
        return datetime.fromtimestamp(value / 1000)
    else:
        return None


def make_enkelvoudiginformatieobject_dataclass(cmis_doc, dataclass, skip_deleted=False):
    if cmis_doc.verwijderd and not skip_deleted:
        # Return None if document is deleted.
        return None

    path = reverse('enkelvoudiginformatieobjecten-detail', kwargs={'version': '1', 'uuid': cmis_doc.versionSeriesId})
    url = f"{settings.HOST_URL}{path}"

    download_link = reverse(
        'enkelvoudiginformatieobjecten-download',
        kwargs={'version': '1', 'uuid': cmis_doc.versionSeriesId}
    )
    download_url = f"{settings.HOST_URL}{download_link}"

    #Versie needs to be an integer. So we need to convert it to an integer.
    versie = cmis_doc.versie
    try:
        int_versie = int(Decimal(versie) * 100)
    except ValueError as e:
        int_versie = 0
    except InvalidOperation:
        int_versie = 0

    cmis_creation_date = cmis_doc.properties["cmis:creationDate"]["value"]

    return dataclass(
        url=url,
        inhoud=download_url,
        creatiedatum=to_date(cmis_doc.creatiedatum),
        ontvangstdatum=to_date(cmis_doc.ontvangstdatum),
        verzenddatum=to_date(cmis_doc.verzenddatum),
        integriteit_datum=to_date(cmis_doc.integriteit_datum),
        ondertekening_datum=to_date(cmis_doc.ondertekening_datum),
        titel=cmis_doc.titel,
        identificatie=cmis_doc.identificatie,
        bronorganisatie=cmis_doc.bronorganisatie,
        vertrouwelijkheidaanduiding=cmis_doc.vertrouwelijkheidaanduiding,
        auteur=cmis_doc.auteur,
        status=cmis_doc.status or "gearchiveerd",  # FIXME -> status must be saved in EIO
        beschrijving=cmis_doc.beschrijving or "",
        indicatie_gebruiksrecht=cmis_doc.indicatie_gebruiksrecht,
        ondertekening_soort=cmis_doc.ondertekening_soort,
        informatieobjecttype=cmis_doc.informatieobjecttype,
        formaat=cmis_doc.formaat or "",
        taal=cmis_doc.taal,
        bestandsnaam=cmis_doc.bestandsnaam or "",
        link=cmis_doc.link or "",
        integriteit_algoritme=cmis_doc.integriteit_algoritme,
        integriteit_waarde=cmis_doc.integriteit_waarde,
        bestandsomvang=cmis_doc.bestandsomvang,
        begin_registratie=to_datetime(cmis_doc.begin_registratie) or to_datetime(cmis_creation_date),
        versie=int_versie,
        locked=bool(cmis_doc.versionSeriesCheckedOutId),
    )


def make_enkelvoudiginformatieobject_dataclass_old(cmis_doc, dataclass, skip_deleted=False):
    properties = cmis_doc.getProperties()

    if properties.get('drc:document__verwijderd') and not skip_deleted:
        # Return None if document is deleted.
        return None

    try:
        inhoud = base64.b64encode(cmis_doc.get_content_stream().read()).decode("utf-8")
    except AssertionError:
        return None
    else:
        logger.error(properties)
        cmis_id = properties.get("cmis:versionSeriesId").split("/")[-1]
        path = reverse('enkelvoudiginformatieobjecten-detail', kwargs={'version': '1', 'uuid': cmis_id})
        url = f"{settings.HOST_URL}{path}"
        download_url = f"{settings.HOST_URL}{reverse('cmis:cmis_download', kwargs={'uuid': cmis_id})}"

        obj_dict = {reverse_mapper(key): value for key, value in properties.items() if reverse_mapper(key)}
        # remove verwijderd
        del obj_dict["verwijderd"]

        # Correct datetimes to dates
        for key, value in obj_dict.items():
            if isinstance(value, datetime):
                obj_dict[key] = value.date()

        # These values are not in alfresco
        obj_dict["url"] = url
        obj_dict["inhoud"] = download_url
        obj_dict["bestandsomvang"] = len(inhoud)
        return dataclass(**obj_dict)


def make_objectinformatieobject_dataclass(cmis_doc, dataclass, skip_deleted=False):
    if cmis_doc.verwijderd and not skip_deleted:
        # Return None if document is deleted.
        return None

    path = reverse('objectinformatieobjecten-detail', kwargs={'version': '1', 'uuid': cmis_doc.versionSeriesId})
    url = f"{settings.HOST_URL}{path}"

    eio_path = reverse('enkelvoudiginformatieobjecten-detail', kwargs={'version': '1', 'uuid': cmis_doc.versionSeriesId})
    eio_url = f"{settings.HOST_URL}{eio_path}"

    return dataclass(
        url=url,
        informatieobject=eio_url,
        object=cmis_doc.object,
        object_type=cmis_doc.object_type,
        aard_relatie=cmis_doc.aard_relatie,
        titel=cmis_doc.connectie__titel,
        beschrijving=cmis_doc.connectie__beschrijving,
        registratiedatum=cmis_doc.registratiedatum,
    )


def make_objectinformatieobject_dataclass_OLD(cmis_doc, dataclass):
    properties = cmis_doc.properties

    obj_dict = {
        reverse_mapper(key, "connection"): value
        for key, value in properties.items()
        if reverse_mapper(key, "connection")
    }

    oio_detail = reverse("objectinformatieobjecten-detail", kwargs={"version": "1", "uuid": cmis_doc.versionSeriesId})
    eio_detail = reverse(
        "enkelvoudiginformatieobjecten-detail",
        kwargs={"version": "1", "uuid": cmis_doc.versionSeriesId}
    )
    url = f"{settings.HOST_URL}{oio_detail}"
    eio_url = f"{settings.HOST_URL}{eio_detail}"

    obj_dict["url"] = url
    obj_dict["informatieobject"] = eio_url

    return dataclass(**obj_dict)



from django.db import models

from solo.models import SingletonModel


class CMISConfig(SingletonModel):
    client_url = models.URLField(
        default="http://localhost:8082/alfresco/api/-default-/public/cmis/versions/1.1/browser"
    )
    client_user = models.CharField(max_length=200, default="admin")
    client_password = models.CharField(max_length=200, default="admin")
    locations = models.ManyToManyField("drc_cmis.CMISFolderLocation")

    def __str__(self):
        return "CMIS Configuration"

    class Meta:
        verbose_name = "CMIS Configuration"


class CMISFolderLocation(models.Model):
    location = models.CharField(max_length=200, default="")

    def __str__(self):
        return self.location



"""
Listen to the notifications that are send by the NRC
"""
from vng_api_common.models import APICredential
from vng_api_common.notifications.handlers import default
from zds_client.client import Client

from drc_cmis.client import CMISDRCClient


class ZakenHandler:
    def __init__(self):
        self.cmis_client = CMISDRCClient()

    def handle(self, data: dict) -> None:
        if data.get("actie") == "create" and data.get("resource") == "zaak":
            zaaktype_url = data.get("kenmerken", {}).get("zaaktype")
            zaak_url = data.get("resource_url")

            if zaak_url and zaaktype_url:
                zaaktype_folder = self.create_zaaktype_folder(zaaktype_url)
                self.create_zaak_folder(zaak_url, zaaktype_url, zaaktype_folder)

    def create_zaaktype_folder(self, zaaktype_url):
        client = Client.from_url(zaaktype_url)
        client.auth = APICredential.get_auth(zaaktype_url)
        client.auth.set_claims(scopes=["zds.scopes.zaaktypes.lezen"])
        zaaktype_data = client.retrieve("zaaktype", url=zaaktype_url)
        return self.cmis_client.get_or_create_zaaktype_folder(zaaktype_data)

    def create_zaak_folder(self, zaak_url, zaaktype_url, zaaktype_folder):
        client = Client.from_url(zaak_url)
        client.auth = APICredential.get_auth(zaak_url)
        client.auth.set_claims(
            scopes=[
                "notificaties.scopes.publiceren",
                "zds.scopes.statussen.toevoegen",
                "zds.scopes.zaken.aanmaken",
                "zds.scopes.zaken.bijwerken",
                "zds.scopes.zaken.geforceerd-bijwerken",
                "zds.scopes.zaken.heropenen",
                "zds.scopes.zaken.lezen",
                "zds.scopes.zaken.verwijderen",
            ],
            zaaktypes=[zaaktype_url],
        )
        zaak_data = client.retrieve("zaak", url=zaak_url)
        self.cmis_client.get_or_create_zaak_folder(zaak_data, zaaktype_folder)


class RoutingHandler:
    def __init__(self, config: dict, default):
        self.config = config
        self.default = default

    def handle(self, message: dict):
        handler = self.config.get(message["kanaal"])
        if handler is not None:
            handler.handle(message)
        else:
            self.default.handle(message)


default = RoutingHandler({"zaken": ZakenHandler()}, default=default)



import logging
from decimal import Decimal

from django.conf import settings
from django.utils import timezone
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from cmislib.exceptions import UpdateConflictException

from .client import CMISDRCClient
from .client.convert import (
    make_enkelvoudiginformatieobject_dataclass,
    make_objectinformatieobject_dataclass
)
from .client.exceptions import (
    CmisUpdateConflictException, DocumentDoesNotExistError,
    DocumentExistsError, DocumentLockConflictException,
    DocumentLockedException, DocumentNotLockedException, GetFirstException
)

logger = logging.getLogger(__name__)


class CMISDRCStorageBackend(import_string(settings.ABSTRACT_BASE_CLASS)):
    """
    This is the backend that is used to store the documents in a CMIS compatible backend.

    This class is based on:
        drc.backend.abstract.BaseDRCStorageBackend
    """
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.cmis_client = CMISDRCClient()

    def create_document(self, data, content):
        """
        Create a new document in the CMIS backend.

        Args:
            data (dict): A dict containing the values returned from the serializer.
            content (BytesStream): The content of the document.

        Returns:
            dataclass: An enkelvoudig informatieobject dataclass.

        Raises:
            BackendException: Raised a backend exception if the document exists.
            BackendException: Raised a backend exception if there is a document with the same name.

        """
        identification = data.pop("identificatie")
        logger.debug(f"CMIS_BACKEND: create_document {identification} start")

        try:
            cmis_doc = self.cmis_client.create_document(identification, data, content)
            dict_doc = make_enkelvoudiginformatieobject_dataclass(cmis_doc, self.eio_dataclass)
            logger.debug(f"CMIS_BACKEND: create_document {identification} successful")
            return dict_doc
        except UpdateConflictException:
            error_message = "Document is niet uniek. Dit kan liggen aan de titel, inhoud of documentnaam"
            raise self.exception_class({None: error_message}, create=True)
        except DocumentExistsError as e:
            raise self.exception_class({None: e.message}, create=True)

    def get_documents(self, page=1, page_size=100, filters=None):
        """
        Fetch all the documents from CMIS backend.

        Args:
            filters (dict or None): A dict with the filters that need to be applied.

        Returns:
            dataclass: A list of enkelvoudig informatieobject dataclass.

        """
        print('get_documents')
        logger.debug(f"CMIS_BACKEND: get_documents start")
        if filters and 'versie' in filters:
            filters['versie'] = self._fix_version(filters['versie'])

        cmis_documents = self.cmis_client.get_cmis_documents(filters=filters, page=page, results_per_page=page_size)
        print(cmis_documents)
        documents_data = []
        for cmis_doc in cmis_documents['results']:
            dict_document = make_enkelvoudiginformatieobject_dataclass(cmis_doc, self.eio_dataclass)
            if dict_document:
                documents_data.append(dict_document)
        print(documents_data)
        paginated_result = self.pagination_dataclass(
            count=cmis_documents['total_count'],
            results=documents_data,
        )

        logger.debug(f"CMIS_BACKEND: get_documents successful")
        return paginated_result

    def get_document(self, uuid, version=None, filters=None):
        """
        Get a document by cmis versionId.

        Args:
            uuid (str): The cmis version id (only the uuid part).

        Returns:
            dataclass: An enkelvoudig informatieobject dataclass.

        Raises:
            BackendException: Raised a backend exception if the document does not exists.

        """
        logger.debug(f"CMIS_BACKEND: get_document {uuid} start")
        version = self._fix_version(version)

        if filters and 'versie' in filters:
            version = self._fix_version(filters['versie'])
            del filters['versie']

        try:
            cmis_document = self.cmis_client.get_cmis_document(uuid, filters=filters)
        except DocumentDoesNotExistError as e:
            raise self.exception_class({None: e.message}, create=True, code='document-does-not-exist')
        else:
            logger.debug(f"CMIS_BACKEND: get_document {uuid} successful")
            if version:
                cmis_document = self._find_version(version, cmis_document)
                if not cmis_document:
                    raise self.exception_class(_("Version does not exists"), update=True, code='version-does-not-exist')
            doc = make_enkelvoudiginformatieobject_dataclass(cmis_document, self.eio_dataclass)
            return doc

    def get_document_content(self, uuid):
        logger.debug(f"CMIS_BACKEND: get_document_content {uuid} start")
        try:
            cmis_document = self.cmis_client.get_cmis_document(uuid)
        except DocumentDoesNotExistError as e:
            raise self.exception_class({None: e.message}, create=True)
        else:
            content = cmis_document.get_content_stream()
            content.seek(0)
            logger.debug(f"CMIS_BACKEND: get_document_content {uuid} successful")
            return content.read(), cmis_document.name

    def update_document(self, uuid, lock, data, content):
        """
        Update the document that is saved in the drc.

        Args:
            uuid (str): The uuid from the document.
            data (dict): A dict with the fields that need to be updated.
            content (BytesStream): The content of the document.

        Returns:
            dataclass: An enkelvoudig informatieobject dataclass.

        Raises:
            BackendException: Raised a backend exception if the document does not exists.

        """
        logger.debug(f"CMIS_BACKEND: update_document {uuid} start")
        try:
            cmis_document = self.cmis_client.update_cmis_document(uuid, lock, data, content)
        except DocumentDoesNotExistError as e:
            raise self.exception_class({None: e.message}, update=True)
        except DocumentNotLockedException as e:
            raise self.exception_class({None: e.message}, update=True, code='not-locked')
        except DocumentLockConflictException as e:
            raise self.exception_class({None: e.message}, update=True, code='wrong-lock')
        else:
            logger.debug(f"CMIS_BACKEND: update_document {uuid} successful")
            return make_enkelvoudiginformatieobject_dataclass(cmis_document, self.eio_dataclass)

    def delete_document(self, uuid):
        """
        A request to delete the document.

        Args:
            uuid (str): The uuid of the document.

        Returns:
            dataclass: An enkelvoudig informatieobject dataclass.

        Raises:
            BackendException: Raised a backend exception if the document does not exists.

        """
        logger.debug(f"CMIS_BACKEND: delete_document {uuid} start")
        try:
            cmis_document = self.cmis_client.delete_cmis_document(uuid)
        except DocumentDoesNotExistError as e:
            raise self.exception_class({None: e.message}, create=True)
        else:
            logger.debug(f"CMIS_BACKEND: delete_document {uuid} successful")
            return make_enkelvoudiginformatieobject_dataclass(cmis_document, self.eio_dataclass, skip_deleted=True)

    def lock_document(self, uuid):
        logger.debug(f"CMIS_BACKEND: lock_document {uuid} start")
        cmis_doc = self.cmis_client.get_cmis_document(uuid)
        try:
            pwc = cmis_doc.checkout()
        except CmisUpdateConflictException as exc:
            raise self.exception_class(detail="Document was already checked out", code="dubble_lock") from exc
        logger.debug(f"CMIS_BACKEND: lock_document {uuid} successful")
        return pwc.versionSeriesCheckedOutId

    def unlock_document(self, uuid, lock, force=False):
        logger.debug(f"CMIS_BACKEND: unlock_document {uuid} start with: {lock}")
        cmis_doc = self.cmis_client.get_cmis_document(uuid)
        pwc = cmis_doc.get_private_working_copy()
        if lock == pwc.versionSeriesCheckedOutId or force:
            new_doc = pwc.checkin("Updated via drc ui")
            logger.debug(f"CMIS_BACKEND: unlock_document {uuid} successful")
            return make_enkelvoudiginformatieobject_dataclass(new_doc, self.eio_dataclass, skip_deleted=True)
        raise self.exception_class(detail=_("Lock did not match"), update=True, code='unlock-failed')

    ################################################################################################
    # Splits #######################################################################################
    ################################################################################################

    def create_document_case_connection(self, data):
        """
        Create the connection between a document and a case folder.

        There are 2 possible paths here,
        1. There is no case connected to the document yet. So the document can be connected to the case.
            1.1. If there is a folder, move the document into the correct folder.
        2. There is a case connected to the document, so we need to create a copy of the document.

        Args:
            data (dict): A dict containing the values returned from the serializer.

        Returns:
            dataclass: An object informatieobject dataclass.

        """
        if not data.get("registratiedatum"):
            data["registratiedatum"] = timezone.now()
        logger.debug(f"CMIS_BACKEND: create_document_case_connection {data['registratiedatum']} start")

        document_id = data.get("informatieobject").split("/")[-1]
        try:
            cmis_doc = self.cmis_client.get_cmis_document(document_id)
            if cmis_doc.object == data.get("object"):
                raise self.exception_class(detail=_("connection is not unique"), code="unique")
        except DocumentDoesNotExistError:
            assert False, "Error hier"

        try:
            folder = self.cmis_client.get_folder_from_case_url(data.get("object"))
        except GetFirstException:
            folder = None
        zaak_url = cmis_doc.object
        logger.debug(f"CMIS_BACKEND_VALUE: {zaak_url}")
        if not zaak_url:
            cmis_doc = self.cmis_client.update_case_connection(cmis_doc.objectId, data)
            if folder:
                self.cmis_client.move_to_case(cmis_doc, folder)
        elif zaak_url == data.get("object"):
            pass
        else:
            cmis_doc = self.cmis_client.copy_document(cmis_doc, folder, data)

        objectinformatieobject = make_objectinformatieobject_dataclass(cmis_doc, self.oio_dataclass)
        logger.debug(f"CMIS_BACKEND: create_document_case_connection {data['registratiedatum']} successful")
        return objectinformatieobject

    def get_document_case_connections(self, filters=None):
        """
        Get all connections between case folders and documents.

        Returns:
            dataclass: A list of object informatieobject dataclass.

        """
        logger.debug(f"CMIS_BACKEND: get_document_case_connections start")

        if not filters:
            filters = {}

        if 'informatieobject' in filters and filters.get('informatieobject'):
            document = self.get_document_case_connection(filters.get('informatieobject').split('/')[-1])
            return [document]

        if 'object' not in filters:
            filters["object"] = "NOT NULL"
        cmis_documents = self.cmis_client.get_cmis_documents(filters=filters, page=0)
        documents_data = []

        for cmis_doc in cmis_documents.get('results'):
            dict_document = make_objectinformatieobject_dataclass(cmis_doc, self.oio_dataclass)
            if dict_document:
                documents_data.append(dict_document)
        logger.debug(f"CMIS_BACKEND: get_document_case_connections successful")
        return documents_data

    def get_document_case_connection(self, uuid):
        """
        Get a single connection by uuid.

        Args:
            uuid (str): the CMIS id from the connected document.

        Returns:
            dataclass: A object informatieobject dataclass.

        Raises:
            BackendException: If the document can not be found.

        """
        logger.debug(f"CMIS_BACKEND: get_document_case_connection {uuid} start")
        try:
            cmis_document = self.cmis_client.get_cmis_document(
                uuid, filters={"drc:connectie__zaakurl": "NOT NULL"}
            )
        except DocumentDoesNotExistError as e:
            raise self.exception_class({None: e.message}, create=True)
        else:
            objectinformatieobject = make_objectinformatieobject_dataclass(cmis_document, self.oio_dataclass)
            logger.debug(f"CMIS_BACKEND: get_document_case_connection {uuid} successful")
            return objectinformatieobject

    def update_document_case_connection(self, uuid, data):
        """
        Update the connection.

        Args:
            uuid (str): The CMIS id of the document.
            data (dict): The data that needs to be updated.

        Returns:
            dataclass: A object informatieobject dataclass.

        Raises:
            BackendException: If the document can not be found.

        """
        logger.debug(f"CMIS_BACKEND: update_document_case_connection {uuid} start")
        try:
            cmis_document = self.cmis_client.update_case_connection(uuid, data)
        except DocumentDoesNotExistError as e:
            raise self.exception_class({None: e.message}, create=True)
        else:
            objectinformatieobject = make_objectinformatieobject_dataclass(cmis_document, self.oio_dataclass)
            logger.debug(f"CMIS_BACKEND: update_document_case_connection {uuid} successful")
            return objectinformatieobject

    def delete_document_case_connection(self, uuid):
        """
        Uncouple a document from the case folder.

        Args:
            uuid (str): The CMIS id of the document.

        Returns:
            dataclass: A object informatieobject dataclass.

        Raises:
            BackendException: If the document can not be found.

        """
        logger.debug(f"CMIS_BACKEND: delete_document_case_connection {uuid} start")
        try:
            cmis_document = self.cmis_client.delete_case_connection(uuid)
        except DocumentDoesNotExistError as e:
            raise self.exception_class({None: e.message}, create=True)
        else:
            objectinformatieobject = make_objectinformatieobject_dataclass(cmis_document, self.oio_dataclass)
            logger.debug(f"CMIS_BACKEND: delete_document_case_connection {uuid} successful")
            return objectinformatieobject

    def _fix_version(self, version):
        if version:
            return Decimal(version) / Decimal('100.0')
        return None

    def _find_version(self, version, cmis_document):
        versions = cmis_document.get_all_versions()
        if int(version) == version:
            version = round(version, 1)
        return versions.get(str(version))



from django.contrib import admin
from django.urls import include, path
from django.views.generic import TemplateView

urlpatterns = [
    path('admin/', admin.site.urls),
    path('test-url/<version>/<uuid>', TemplateView.as_view(), name='enkelvoudiginformatieobjecten-detail'),
    path('test-url/<version>/<uuid>/download/', TemplateView.as_view(), name='enkelvoudiginformatieobjecten-download'),
    path('test-url2/<version>/<uuid>', TemplateView.as_view(), name='objectinformatieobjecten-detail'),
    path('ref/', include('vng_api_common.notifications.urls')),
]



import logging
from json.decoder import JSONDecodeError

import requests

from drc_cmis.client.exceptions import (
    CmisBaseException, CmisInvalidArgumentException, CmisNotSupportedException,
    CmisNoValidResponse, CmisObjectNotFoundException,
    CmisPermissionDeniedException, CmisRuntimeException,
    CmisUpdateConflictException, GetFirstException
)

logger = logging.getLogger(__name__)


class CMISRequest:
    def __init__(self):
        from drc_cmis.models import CMISConfig
        config = CMISConfig.get_solo()

        self.base_url = config.client_url
        self.root_folder_url = f"{self.base_url}/root"
        self.user = config.client_user
        self.password = config.client_password

    def get_request(self, url, params=None):
        logger.debug(f"GET: {url} | {params}")
        headers = {'Accept': 'application/json'}
        response = requests.get(url, params=params, auth=(self.user, self.password), headers=headers)
        if not response.ok:
            raise Exception('Error with the query')

        if response.headers.get('Content-Type').startswith('application/json'):
            return response.json()
        return response.content

    def post_request(self, url, data, files=None):
        logger.debug(f"POST: {url} | {data}")
        headers = {'Accept': 'application/json'}
        response = requests.post(url, data=data, auth=(self.user, self.password), files=files, headers=headers)
        if not response.ok:
            error = response.json()
            if response.status_code == 401:
                raise CmisPermissionDeniedException(
                    status=response.status_code, url=url, message=error.get('message'), code=error.get('exception')
                )
            elif response.status_code == 400:
                raise CmisInvalidArgumentException(
                    status=response.status_code, url=url, message=error.get('message'), code=error.get('exception')
                )
            elif response.status_code == 404:
                raise CmisObjectNotFoundException(
                    status=response.status_code, url=url, message=error.get('message'), code=error.get('exception')
                )
            elif response.status_code == 403:
                raise CmisPermissionDeniedException(
                    status=response.status_code, url=url, message=error.get('message'), code=error.get('exception')
                )
            elif response.status_code == 405:
                raise CmisNotSupportedException(
                    status=response.status_code, url=url, message=error.get('message'), code=error.get('exception')
                )
            elif response.status_code == 409:
                raise CmisUpdateConflictException(
                    status=response.status_code, url=url, message=error.get('message'), code=error.get('exception')
                )
            elif response.status_code == 500:
                raise CmisRuntimeException(
                    status=response.status_code, url=url, message=error.get('message'), code=error.get('exception')
                )
            else:
                raise CmisBaseException(
                    status=response.status_code, url=url, message=error.get('message'), code=error.get('exception')
                )

        try:
            return response.json()
        except JSONDecodeError:
            if not response.text:
                return None
            raise CmisNoValidResponse(
                status=response.status_code, url=url, message=response.text, code="invalid_response"
            )

    def get_first_result(self, json, return_type):
        if len(json.get('results')) == 0:
            raise GetFirstException()

        return return_type(json.get('results')[0])

    def get_all_resutls(self, json, return_type):
        results = []
        for item in json.get('results'):
            results.append(return_type(item))
        return results

    def get_all_objects(self, json, return_type):
        objects = []
        for item in json:
            objects.append(return_type(item.get('object')))
        return objects



ZAAKTYPE_MAP = {"url": "drc:zaaktype__url", "identificatie": "drc:zaaktype__identificatie"}

ZAAK_MAP = {
    "url": "drc:zaak__url",
    "identificatie": "drc:zaak__identificatie",
    "zaaktype": "drc:zaak__zaaktypeurl",
    "bronorganisatie": "drc:zaak__bronorganisatie",
}


DOCUMENT_MAP = {
    # "uuid": "drc:",
    "identificatie": "drc:document__identificatie",
    "bronorganisatie": "drc:document__bronorganisatie",
    "creatiedatum": "drc:document__creatiedatum",
    "titel": "drc:document__titel",
    "vertrouwelijkheidaanduiding": "drc:document__vertrouwelijkaanduiding",
    "auteur": "drc:document__auteur",
    "status": "drc:document__status",
    "beschrijving": "drc:document__beschrijving",
    "ontvangstdatum": "drc:document__ontvangstdatum",
    "verzenddatum": "drc:document__verzenddatum",
    "indicatie_gebruiksrecht": "drc:document__indicatiegebruiksrecht",
    "ondertekening_soort": "drc:document__ondertekeningsoort",
    "ondertekening_datum": "drc:document__ondertekeningdatum",
    "informatieobjecttype": "drc:document__informatieobjecttype",
    "formaat": "drc:document__formaat",
    "taal": "drc:document__taal",
    "bestandsnaam": "drc:document__bestandsnaam",
    "bestandsomvang": "cmis:contentStreamLength",
    "versie": "cmis:versionLabel",
    # "inhoud": "drc:",
    "link": "drc:document__link",
    "integriteit_algoritme": "drc:document__integriteitalgoritme",
    "integriteit_waarde": "drc:document__integriteitwaarde",
    "integriteit_datum": "drc:document__integriteitdatum",
    "verwijderd": "drc:document__verwijderd",
    "begin_registratie": "drc:document__begin_registratie",
}

CONNECTION_MAP = {
    "object": "drc:connectie__zaakurl",
    "object_type": "drc:connectie__objecttype",
    "aard_relatie": "drc:connectie__aardrelatieweergave",
    "titel": "drc:connectie__titel",
    "beschrijving": "drc:connectie__beschrijving",
    "registratiedatum": "drc:connectie__registratiedatum",
}

REVERSE_ZAAKTYPE_MAP = {value: key for key, value in ZAAKTYPE_MAP.items()}
REVERSE_ZAAK_MAP = {value: key for key, value in ZAAK_MAP.items()}
REVERSE_DOCUMENT_MAP = {value: key for key, value in DOCUMENT_MAP.items()}
REVERSE_CONNECTION_MAP = {value: key for key, value in CONNECTION_MAP.items()}


def mapper(drc_name, type="document"):
    if type == "document":
        return DOCUMENT_MAP.get(drc_name, None)
    if type == "connection":
        return CONNECTION_MAP.get(drc_name, None)
    if type == "zaaktype":
        return ZAAKTYPE_MAP.get(drc_name, None)
    if type == "zaak":
        return ZAAK_MAP.get(drc_name, None)
    return None


def reverse_mapper(cmis_name, type="document"):
    if type == "document":
        return REVERSE_DOCUMENT_MAP.get(cmis_name, None)
    if type == "connection":
        return REVERSE_CONNECTION_MAP.get(cmis_name, None)
    if type == "zaaktype":
        return REVERSE_ZAAKTYPE_MAP.get(cmis_name, None)
    if type == "zaak":
        return REVERSE_ZAAK_MAP.get(cmis_name, None)
    return None



# Generated by Django 2.2.2 on 2019-10-29 13:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('drc_cmis', '0002_auto_20190523_1054'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cmisconfig',
            name='client_url',
            field=models.URLField(default='http://localhost:8082/alfresco/api/-default-/public/cmis/versions/1.1/browser'),
        ),
    ]



from dataclasses import dataclass
from datetime import date, datetime


@dataclass
class PaginationObject:
    count: int
    results: list
    next: str = None
    previous: str = None


@dataclass
class EnkelvoudigInformatieObject:
    url: str
    inhoud: str
    creatiedatum: date
    ontvangstdatum: date
    verzenddatum: date
    integriteit_datum: date
    ondertekening_datum: date
    titel: str
    identificatie: str
    bronorganisatie: str
    vertrouwelijkheidaanduiding: str
    auteur: str
    status: str
    beschrijving: str
    indicatie_gebruiksrecht: str
    ondertekening_soort: str
    informatieobjecttype: str
    formaat: str
    taal: str
    bestandsnaam: str
    link: str
    integriteit_algoritme: str
    integriteit_waarde: str
    bestandsomvang: str
    begin_registratie: datetime
    versie: str
    locked: bool


@dataclass
class ObjectInformatieObject:
    url: str
    informatieobject: str
    object: str
    object_type: str
    aard_relatie: str
    titel: str
    beschrijving: str
    registratiedatum: str



import sys

from django.conf import settings


class _Settings(object):
    @property
    def BASE_FOLDER_LOCATION(self):
        return getattr(settings, "BASE_FOLDER_LOCATION", "DRC")

    def __getattr__(self, name):
        return globals()[name]


# other parts of itun that you WANT to code in
# module-ish ways
sys.modules[__name__] = _Settings()
